// WARNING:  this file is automatically included by Vector.h.
//           DO NOT MAKE THIS .CPP FILE ITS OWN COMPILATION UNIT.
//
//  Modified by Curtis Krauskopf (the Database Managers, Inc., www.decompile.com)
//  for wider distribution and to apply modern programming practices while trying
//  to maintain the spirit of the original library.

// CDK:  added throw() specifiers:  July 1, 2006


template <class Etype>
Java::Vector<Etype>::Vector( unsigned int initialCapacity, unsigned int capacityIncrement )
{
    m_size = 0;
    m_capacity = initialCapacity;
    m_pData = new Etype*[ m_capacity ];

    m_increment = capacityIncrement;
}

template <class Etype>
Java::Vector<Etype>::Vector( const Vector<Etype>& rhv )
{
    m_size = rhv.m_size;
    m_capacity = rhv.m_capacity;
    m_pData = new Etype*[ m_capacity ];
    m_increment = rhv.m_increment;


    for( unsigned int i = 0; i < m_size; i++ )
    {
        m_pData[i] = new Etype( *(rhv.m_pData[i]) );
    }
}

template <class Etype>
Java::Vector<Etype>::~Vector()
{
    removeAllElements();
    delete [] m_pData;
}


template <class Etype>
unsigned int
Java::Vector<Etype>::capacity() const
{
    return m_capacity;
}

template <class Etype>
bool
Java::Vector<Etype>::contains( const Etype &elem ) const
{
    for ( unsigned int i = 0; i < m_size; i++ )
    {
        if ( *m_pData[i] == elem )
            return true;
    }

    return false;
}

template <class Etype>
void
Java::Vector<Etype>::copyInto( Etype* array ) const
{
    for( unsigned int i = 0; i < m_size; i++ )
        array[i] = *m_pData[i];
}


template <class Etype>
const Etype &
Java::Vector<Etype>::elementAt( unsigned int index ) const
  throw (char *)
{
    verifyIndex( index );
    return *m_pData[index];
}

template <class Etype>
const Etype &
Java::Vector<Etype>::firstElement() const
throw(char *)
{
    if ( m_size == 0 )
    {
#ifdef JAVALIB_USE_CERR
        cerr << "firstElement() called on empty Vector" << endl;
        exit(1);
#else
        throw "Empty Vector Exception";
#endif
    }

    return *m_pData[ 0 ];
}

template <class Etype>
int
Java::Vector<Etype>::indexOf( const Etype &elem ) const
{
    for ( unsigned int i = 0; i < m_size; i++ )
    {
        if ( *m_pData[ i ] == elem )
            return i;
    }

    return -1;
}

template <class Etype>
bool
Java::Vector<Etype>::isEmpty() const
{
    return m_size == 0;
}

template <class Etype>
const Etype &
Java::Vector<Etype>::lastElement() const
  throw (char *)
{
    if ( m_size == 0 )
    {
#ifdef JAVALIB_USE_CERR
        cerr << "lastElement() called on empty Vector" << endl;
        exit(1);
#else
        throw "Empty Vector Exception";
#endif
    }

    return *m_pData[ m_size - 1 ];
}

template <class Etype>
int
Java::Vector<Etype>::lastIndexOf( const Etype &elem ) const
{
    //check for empty vector
    if ( m_size == 0 )
        return -1;

    unsigned int i = m_size;

    do
    {
        i -= 1;
        if ( *m_pData[i] == elem )
            return i;

    }
    while ( i != 0 );

    return -1;
}

template <class Etype>
unsigned int
Java::Vector<Etype>::size() const
{
    return m_size;
}

template <class Etype>
void
Java::Vector<Etype>::addElement( const Etype &obj )
{
    if ( m_size == m_capacity )
        ensureCapacity( m_capacity + m_increment );

    m_pData[ m_size++ ] = new Etype( obj );
}

template <class Etype>
void
Java::Vector<Etype>::ensureCapacity( unsigned int minCapacity )
{
    if ( minCapacity > m_capacity )
    {
        unsigned int i;
        m_capacity = minCapacity;
        Etype** temp = new Etype*[ m_capacity ];


        //copy all the elements over upto newsize
        memcpy( temp, m_pData, sizeof(Etype*) * m_size );
        //for ( i = 0; i < m_size; i++ )
        //    temp[i] = m_pData[i];


        delete [] m_pData;
        m_pData = temp;

    }
}

template <class Etype>
void
Java::Vector<Etype>::insertElementAt( const Etype &obj, unsigned int index )
  throw (char *)
{
    if ( index == m_size )
        addElement( obj );
    else
    {
        verifyIndex( index );    //this will throw if true

        if ( m_size == m_capacity )
            ensureCapacity( m_capacity + m_increment);

        Etype* newItem = new Etype(obj);    //pointer to new item
        Etype* tmp;                            //temp to hold item to be moved over.

        for( unsigned int i = index; i <= m_size; i++ )
        {
            tmp = m_pData[i];
            m_pData[i] = newItem;

            if ( i != m_size )
                newItem = tmp;
            else
                break;
        }
    }
    m_size++;
}



template <class Etype>
const Etype&
Java::Vector<Etype>::remove( unsigned int index )
  throw (char *)          //modifier
{
  Etype retval = get(index);
  removeElementAt(index);
  return retval;
}


template <class Etype>
void
Java::Vector<Etype>::removeAllElements()
{
    //avoid memory leak
    for ( unsigned int i = 0; i < m_size; i++ )
        delete m_pData[i];

    m_size = 0;
}

template <class Etype>
bool
Java::Vector<Etype>::removeElement( const Etype &obj )
{
    for ( unsigned int i = 0; i < m_size; i++ )
    {
        if ( *m_pData[i] == obj )
        {
            removeElementAt( i );
            return true;
        }
    }

    return false;
}

template <class Etype>
void
Java::Vector<Etype>::removeElementAt( unsigned int index )
  throw (char *)
{
    verifyIndex( index );

    delete m_pData[ index ];

    unsigned int i;
    for ( i = index+1; i < m_size; i++ )
        m_pData[ i - 1 ] = m_pData[ i ];

    m_pData[i];
    m_size--;
}

template <class Etype>
void
Java::Vector<Etype>::setElementAt( const Etype &obj, unsigned int index )
  throw (char *)
{
    verifyIndex( index );
    
    *m_pData[ index ] = obj;
}

template <class Etype>
void
Java::Vector<Etype>::setSize( unsigned int newSize )
{
    if ( newSize > m_capacity )
        ensureCapacity( newSize );
    else if ( newSize < m_size )
    {
        for( unsigned int i = newSize; i < m_size; i++ )
            delete m_pData[i];

        m_size = newSize;
    }
}

template <class Etype>
void
Java::Vector<Etype>::trimToSize()
{
    if ( m_size != m_capacity )
    {
        Etype** temp = new Etype*[ m_size ];
        unsigned int i;

        for ( i = 0; i < m_size; i++ )
            temp[i] = m_pData[i];

        delete [] m_pData;

        m_pData = temp;
        m_capacity = m_size;
    }
}

template <class Etype>
int
Java::Vector<Etype>::min( unsigned int left, unsigned int right ) const
{
    return left < right ? left : right;
}

template <class Etype>
void
Java::Vector<Etype>::verifyIndex( unsigned int index ) const
  throw (char *)
{
    if ( index >= m_size )
    {
#ifdef JAVALIB_USE_CERR
        cerr << "Index Out Of Bounds";
        exit(1);
#else
        throw "Index Out Of Bounds";
#endif
    }
}

template <class Etype>
const Etype &
Java::Vector<Etype>::operator[]( unsigned int index ) const
{
    return elementAt( index );
}

template <class Etype>
Etype &
Java::Vector<Etype>::operator[]( unsigned int index )
  throw (char *)
{
    verifyIndex( index );
    return *m_pData[ index ];
}




